<?php

use Slim\Http\Request;
use Slim\Http\Response;

// Open Routes
$app->get('/books', function (Request $request, Response $response, array $args) {
    $args['user'] = $this->auth->getAuthenticatedUser();
    $args['allBooks'] = $this->book->getAllBooks();
    $args['vote'] = $this->vote;
    return $this->renderer->render($response, 'books.php', $args);
});
$app->get('/', function (Request $request, Response $response, array $args) {
    return $this->renderer->render($response, 'index.php', $args);
});
$app->get('/login', function (Request $request, Response $response, array $args) {
    return $this->renderer->render($response, 'login.php', $args);
});
$app->post('/login', function (Request $request, Response $response, array $args) {
    $form = $request->getParsedBody();
    $user = $this->user->findUserByUsername($form['username']);
    
    if (empty($user)) {
        $this->session->getFlashBag()->add('error', 'Username was not found');
        return $response->withRedirect('/login');
    }

    if (!password_verify($form['password'], $user['password'])) {
        $this->session->getFlashBag()->add('error', 'Invalid Password');
        return $response->withRedirect('/login');
    }
    
    $this->auth->saveUserSession($user);
    return $response->withRedirect('/');
});
$app->get('/logout', function (Request $request, Response $response, array $args) {
    $this->session->remove('auth_logged_in');
    $this->session->remove('auth_user_id');
    $this->session->remove('auth_roles');

    $this->session->getFlashBag()->add('success', 'Successfully Logged Out');
    return $response->withRedirect('/login');
});
$app->get('/register', function (Request $request, Response $response, array $args) {
    return $this->renderer->render($response, 'register.php', $args);
});
$app->post('/register', function (Request $request, Response $response, array $args) {
    $form = $request->getParsedBody();
    var_dump($form);

    if ($form['password'] != $form['confirm_password']) {
        $this->session->getFlashBag()->add('error', 'Passwords do NOT match');
        return $response->withRedirect('/register');
    }

    $user = $this->user->findUserByUsername($form['username']);
    if (!empty($user)) {
        $this->session->getFlashBag()->add('error', 'User Already Exists');
        return $response->withRedirect('/register');
    }

    $hashed = password_hash($form['password'], PASSWORD_DEFAULT);
    $user = $this->user->createUser($form['username'], $hashed);
    $this->auth->saveUserSession($user);
    $this->session->getFlashBag()->add('success', 'User Added');
    return $response->withRedirect('/');
});

// Authenticated User
$app->group('', function(Slim\App $app) {
    $app->get('/account', function (Request $request, Response $response, array $args) {
        $form = $request->getParsedBody();
        return $this->renderer->render($response, 'account.php', $args);
    });
    $app->post('/account', function (Request $request, Response $response, array $args) {
        $form = $request->getParsedBody();

        if ($form['password'] != $form['confirm_password']) {
            $this->session->getFlashBag()->add('error', 'New passwords do not match. Please try again.');
            return $response->withRedirect('/account');
        }
        
        $user = $this->auth->getAuthenticatedUser();
        
        if (empty($user)) {
            $this->session->getFlashBag()->add('error', 'Some Error Happend. Try again. If it continues,  please log out and back in.');
            return $response->withRedirect('/account');
        }
        
        if (!password_verify($form['current_password'], $user['password'])) {
            $this->session->getFlashBag()->add('error', 'Current password was incorrect, please try again');
            return $response->withRedirect('/account');
        }
        
        $hashed = password_hash($form['password'], PASSWORD_DEFAULT);
        
        if (!$this->user->updatePassword($hashed, $user['id'])) {
            $this->session->getFlashBag()->add('error', 'Could not update password, please try again.');
            return $response->withRedirect('/account');
        }
        
        $this->session->getFlashBag()->add('success', 'Password Updated');
        return $response->withRedirect('/account');
    });
    $app->get('/add', function (Request $request, Response $response, array $args) {
        return $this->renderer->render($response, 'add.php', $args);
    });
    $app->post('/add', function (Request $request, Response $response, array $args) {
        $book = $request->getParsedBody();

        if ($this->book->addBook($book['title'], $book['description'], $this->session->get('auth_user_id'))) {
            $this->session->getFlashBag()->add('success', 'Book Added');
            return $response->withRedirect('/books');
        }
        $this->session->getFlashBag()->add('error', 'Unable to Add Book');
        return $response->withRedirect('/add');
    });
    $app->get('/vote', function (Request $request, Response $response, array $args) {
        $user = $this->auth->getAuthenticatedUser();

        $this->vote->vote(
            $request->getQueryParam('bookId'), 
            $request->getQueryParam('vote'), 
            $user['id']
        );
        
        return $response->withRedirect('/books');
    });
})->add(new App\Middleware\AuthMiddleware($container));

// Administrators
$app->group('', function(Slim\App $app) {
    $app->get('/admin', function (Request $request, Response $response, array $args) {
        $args['allUsers'] = $this->user->getAllUsers();
        return $this->renderer->render($response, 'admin.php', $args);
    });
    $app->get('/role/{userId}/{roleId}', function (Request $request, Response $response, array $args) {
        $user = $this->user->changeRole($args['userId'], $args['roleId']);
        
        if ($user['role_id'] == 1) {
            $this->session->getFlashBag()->add('success', $user['username'] . ' promoted to admin!');
        } else {
            $this->session->getFlashBag()->add('success', $user['username'] . ' demoted to restricted user!');
        }
        
        return $response->withRedirect('/admin');
    });
})->add(new App\Middleware\AdminMiddleware($container));

// Authorized Owners
$app->group('', function(Slim\App $app) {
    $app->get('/delete/{bookId}', function (Request $request, Response $response, array $args) {
        $book = $this->book->getBook($args['bookId']);
        if ($this->book->deleteBook($book['id'])) {
            $this->session->getFlashBag()->add('success', 'Book Deleted');
        } else {
            $this->session->getFlashBag()->add('error', 'Unable to Delete Book');
        }
        return $response->withRedirect('/books');
    });
    $app->get('/edit/{bookId}', function (Request $request, Response $response, array $args) {
        $args['book'] = $this->book->getBook($args['bookId']);
        $args['buttonText'] = 'Update Book';
        return $this->renderer->render($response, 'edit.php', $args);
    });
    $app->post('/edit/{bookId}', function (Request $request, Response $response, array $args) {
        $book = $this->book->getBook($args['bookId']);
        $form = $request->getParsedBody();

        if ($this->book->updateBook($book['id'], $form['title'], $form['description'])) {
            $this->session->getFlashBag()->add('success', 'Book Updated');
            return $response->withRedirect('/books');
        }

        $this->session->getFlashBag()->add('error', 'Unable to Update Book');
        return $response->withRedirect('/edit/'.$book['id']);
    });
})->add(new App\Middleware\OwnerMiddleware($container));